// Code generated by command: go run gen.go -out ../md5block_amd64.s -stubs ../md5block_amd64.go -pkg=md5simd. DO NOT EDIT.

// +build !appengine
// +build !noasm
// +build gc

package md5simd

// Encode p to digest
//go:noescape
func blockScalar(dig *[4]uint32, p []byte)
